#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import yaml
import re

__all__ = ['config']


class _Config(object):
    def __init__(self):
        self._CONFIG_PATH = os.path.abspath(os.path.expanduser('~/.tvm.yaml'))
        self._config = self._set_safe_defaults()

        # Overrides safe_defaults if values are provided.
        self._read_config()
        self._validate_config()

    def _set_safe_defaults(self):
        return {
            'record_storage_path': '~/sample',
            'physical_storage_paths': ['~/Downloads/tvm-downloads'],
            'cache_path': '~/.tvm-cache/',
            'ignore_paths': ['Thumbs', '.DS_Store', '^\._.*'],
            'ignore_titles': []
        }

    def _read_config(self):
        try:
            with open(self._CONFIG_PATH, 'rU') as stream:
                configuration = yaml.load(stream)

            for key, item in configuration.iteritems():
                self._config[key] = item

        except IOError:
            print('\033[34m~>\033[0m\033[39m `.tvm.yaml` file not found at $HOME. Using safe defaults.')

    def _validate_config(self):
        EXPECTED_CONFIG_KEYS = [
            'record_storage_path',
            'physical_storage_paths',
            'cache_path',
            'ignore_paths'
        ]

        for key in EXPECTED_CONFIG_KEYS:
            if self._config[key]: continue
            print('WARN! No value was assigned to the key, "%s".' % key)

    @property
    def record_storage_path(self):
        storage_path = self._config['record_storage_path'] if self._config['record_storage_path'] else '~/sample'
        return os.path.abspath(os.path.expanduser(storage_path))

    @property
    def physical_storage_paths(self):
        storage_path = self._config['physical_storage_paths']
        return map(lambda p: os.path.abspath(os.path.expanduser(p)), storage_path)

    @property
    def last_notification_path(self):
        return os.path.join(self.cache_path, '.last_notification.json')

    @property
    def added_path(self):
        return os.path.join(self.record_storage_path, 'shows_added.csv')

    @property
    def viewed_path(self):
        return os.path.join(self.record_storage_path, 'shows_viewed.csv')

    @property
    def ignore_pattern(self):
        ignore_paths = self._config['ignore_paths'] if self._config['ignore_paths'] else []
        return re.compile('|'.join(ignore_paths))

    @property
    def ignore_titles(self):
        return self._config['ignore_titles']

    @property
    def api_keys(self):
        return self._config['api_keys']

    @property
    def cache_path(self):
        return os.path.abspath(os.path.expanduser(self._config['cache_path']))


# Relying on the fact that modules are loaded once for this to be a singleton.
config = _Config()
