#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time
import os

from datetime import datetime


def str_to_datetime(date):
    """Given a date as a string, convert it into a :class:`datetime.datetime` object.

    :param str date: The string that needs to be converted.
    :returns: A :class:`datetime.datetime` object.
    :rtype: :class:`datetime.datetime`
    """
    return None if not date else datetime.strptime(date, '%Y-%m-%d %H:%M:%S')


def datetime_to_str(date):
    """Given a :class:`datetime.datetime`, convert it into a string.

    :param date: The :class:`datetime.datetime` object to be converted.
    :type date: :class:`datetime.datetime`
    :returns: The datetime as a string.
    :rtype: str
    """
    return '' if not date else date.strftime('%Y-%m-%d %H:%M:%S')


def date_to_datetime(date):
    """Given a :class:`datetime.date` object, convert it into a :class:`datetime.datetime` object.

    :param date: The :class:`datetime.date` object to be converted.
    :type date: :class:`datetime.date`
    :returns: A :class:`datetime.datetime` object.
    :rtype: :class:`datetime.datetime`
    """
    return None if not date else datetime.combine(date, datetime.min.time())


def datetime_to_time(d_time):
    """Given a :class:`datetime.datetime`, convert it into a :class:`datetime.time` object.

    :param d_time: The :class:`datetime.datetime` object to be converted.
    :type d_time: :class:`datetime.datetime`
    :returns: The extracted `time` from :class:`datetime.time`.
    :rtype: :class:`datetime.time`
    """
    return None if not d_time else d_time.time()


def time_to_str(t_time):
    """Given a :class:`time` object, convert it into a string.

    :param t_time: The :class:`time` object.
    :type t_time: :class:`time`
    :returns: The hour and minutes in string form.
    :rtype: str
    """
    return None if not t_time else t_time.strftime('%H:%M')


def time_now_str():
    """Take the current time and convert it into a string.

    :returns: The current time in string form.
    :rtype: str
    """
    return time_to_str(datetime_to_time(datetime.now()))


def c_datetime(path):
    """Given a valid file `path`, return the created datetime.

    :params str path: An absolute path to the file.
    :returns: The file created datetime.
    :rtype: :class:`datetime.datetime`
    """
    return datetime.strptime(time.ctime(os.path.getmtime(path)), '%a %b %d %H:%M:%S %Y')


def pretty_date(d_time):
    """Given a datetime object, convert it into a "pretty", human readable string.

    :param d_time: The :class:`datetime.datetime` object.
    :type d_time: :class:`datetime.datetime`
    :returns: A human readable string.
    :rtype: str
    """
    if not d_time:
        return '-'

    if isinstance(d_time, str):
        return d_time

    diff = datetime.now() - d_time
    second_diff = diff.seconds
    day_diff = diff.days

    if day_diff == 0:
        if second_diff < 10:
            return 'just now'
        if second_diff < 60:
            return str(second_diff) + ' seconds ago'
        if second_diff < 120:
            return 'a minute ago'
        if second_diff < 3600:
            return str(int(round(second_diff / 60.0))) + ' minutes ago'
        if second_diff < 7200:
            return 'an hour ago'
        if second_diff < 86400:
            return str(int(round(second_diff / 3600.0))) + ' hours ago'

    return d_time.strftime('%d %b %Y')
