#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import sys
import subprocess

from tvm.api.config import config


def print_errors(errors, terminate_program=True):
    """Prints errors to the console.

    :param dict errors: A dictionary of errors.
    :param bool terminate_program: (optional) Set the True to `exit(0)`.

    .. note::
        The `errors` dict is expected to be in the form: `{'error_type': [defect, ], }`.
    """
    # Sort the defects before printing them so it's easier on the eyes.
    for error_type, defects in errors.iteritems():
        if not isinstance(defects, str):
            # Sort by title, season and episode.
            errors[error_type] = sorted(defects, key=lambda d: (d[-1], d[0], d[1]))

            # Convert each list into a string delimited by a comma.
            errors[error_type] = map(lambda d: ','.join(d), errors[error_type])

    error_count = 0
    for error_type, defects in errors.iteritems():
        error_count += len(defects)

        for defect in defects:
            defect = ','.join(defect) if isinstance(defect, list) else defect
            warn('(error) "%s", (defective record) "%s"' % (error_type, defect))

    if error_count > 0:
        warn('There are %d total errors found.' % error_count)
    terminate_program and error_count > 0 and exit(0)


def print_matched_status(matched):
    """Prints multiple matches to the console. Use this function when the search
    query results in multiple TV show titles.

    To find matches, refer to :mod:`tvm.tvcollection.find_matching_titles`.

    :param list matched: A list of possible matches.

    .. note::
        The `matched` argument is expected to be `[(percentage, title), ]`.
    """
    if not matched:
        info('No TV shows matching your expression.')
        exit(0)

    if isinstance(matched, list):  # Multiple matches.
        info('Do you mean any of these titles?')
        for match in matched:
            title, ratio, percentage = match
            info2('(%s%%) %s' % (percentage, title))
        exit(0)

    info('Found a match! "%s"' % matched)


def build_episode_path(entry):
    """Builds and returns the absolute path to an episode on disk. """
    season, episode, title = entry
    for path in config.physical_storage_paths:
        possible_path = os.path.join(path, '/'.join([title, season]))

        # Get the file extension.
        for root, _, files in os.walk(possible_path):
            for file in files:
                episode_number = os.path.splitext(file)[0]
                if episode_number != episode:
                    continue

                return os.path.join(possible_path, file)
    return None


def open_with_default(file_path):
    """Opens a file given the `file_path` using the default application.

    :param str file_path: Absolute file path to the episode to open.
    :returns: True if the file was successfully opened, False otherwise.
    :rtype: bool
    """
    try:
        if sys.platform == 'darwin':  # Mac OS.
            subprocess.call(['open', file_path])
        elif sys.platform == 'win32':  # Windows 32bit.
            os.startfile(file_path)  # os.startfile only available on Windows.
        elif sys.platform.startswith('linux'):
            subprocess.call(['xdg-open', file_path])
        else:
            err('Unexpected platform "%s". Submit a bug on https://github.com/davidvuong/tvm.' % sys.platform)
            return False

        return True
    except OSError:
        err('Cannot open path because your platform "%s" is not supported.' % sys.platform)

    return False


def get_last_line(file_name):
    process = subprocess.Popen(['tail', '-n-1', file_name], stdout=subprocess.PIPE)
    row = process.stdout.read().strip('\n').split(',')[1:]
    return row[0], row[1], row[2].strip('"')


def to_utf8(message):
    try:
        return message.encode('utf-8')
    except UnicodeDecodeError:
        return message


# Reference: http://pueblo.sourceforge.net/doc/manual/ansi_color_codes.html
def err(error_message):
    print("\033[1;31m✖ ERR!\033[0m " + to_utf8(error_message), file=sys.stderr)


def warn(warning_message):
    print("\033[33mWARN!\033[0m " + to_utf8(warning_message), file=sys.stderr)


def info(info_message, end='\n'):
    print("\033[34m~>\033[0m\033[39m " + to_utf8(info_message) + "\033[0m", file=sys.stdout, end=end)


def info2(info_message, end='\n'):
    print("\033[35m⇌   \033[0m\033[39m" + to_utf8(info_message) + "\033[0m", file=sys.stdout, end=end)


def ok(ok_message):
    print("\033[32m✓ OK.\033[0m " + to_utf8(ok_message), file=sys.stdout)


if __name__ == '__main__':
    err("An error has occurred and something bad is happening.")
    warn("Warnings can be ignored if you want to risk what might happen.")
    info("General status information about what's happening.")
    info2("Some additional information accompanying the above info call.")
    ok("Everything looks good. Good to go!")
