#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Proxy for showRSS & perhaps other RSS feeds in the future. """
from __future__ import print_function

import os
import time
import urllib2
import re
import xml.etree.ElementTree as ET

from copy import deepcopy
from difflib import SequenceMatcher

from tvm.api.config import config
from tvm.api.io import err

__all__ = ['RSSSource', 'create_rss']


class RSSSource(object):
    def __init__(self, collection, updates=None):
        """This class should be used to fetch and store new updates ShowRSS. Upon
        instantiation, `RSSSource` will attempt to request for new updates. To avoid
        this behaviour, you can provide a pre-fetched update via `updates`.

        :param collection: An instance of the :class:`TVCollection` class.
        :type collection: :class:`TVCollection`
        :param dict updates: (optional) Provide a dictionary of pre-fetched updates to avoid
            a requesting for a new update.
        """
        self.collection = collection
        self.RSS_ENDPOINT = 'http://showrss.info/'

        # Groups: (season, episode, video_quality).
        self._pattern_1 = re.compile(r'(\d+)x([0-9-]+)(?:.*\s(\d{3,}?))?', flags=re.IGNORECASE)

        # Groups: (date, video_quality).
        self._pattern_2 = re.compile(r'(\d+-\d+-\d+)(?:.*\s(\d{3,}?))?', flags=re.IGNORECASE)

        if not updates:
            self._fetch_updates()
            self._parse_tree()
            self._cleanse_updates()
        else:
            # NOTE: Don't worry about fetching if we're passing in a cached version.
            #       However, because of this raw_updates won't be available.
            self._cleansed_updates = updates
            self._raw_updates = updates

    def _fetch_updates(self):
        try:
            url = os.path.join(self.RSS_ENDPOINT, 'feeds/all.rss')
            response = urllib2.urlopen(url)
            self._root = ET.fromstring(response.read())
        except urllib2.URLError:
            err('Encountered URLError. No internet connection?')
            exit(0)
        except urllib2.HTTPError:
            err('Encountered HTTPError. Invalid RSS URL?')
            exit(0)

    def _extract_from_title(self, title):
        """ Have to parse the title to get the season, episode and video quality. """
        pattern_1_result = self._pattern_1.search(title)
        if pattern_1_result:
            season, episode, quality = pattern_1_result.groups()
            season = int(season)
            try:
                episode = int(episode)
            except ValueError:
                # NOTE: This is needed to account for episodes like '21-22' or worse, '01-02'.
                #       They are then mapped to '21;22' and '1;2'.
                episode = ';'.join(map(str, map(int, episode.split('-'))))

            return season, episode, quality

        pattern_2_result = self._pattern_2.search(title)
        if pattern_2_result:  # Use the date as the episode number and the year as the season.
            date, quality = pattern_2_result.groups()
            date = time.strptime(date, "%Y-%m-%d")
            return date.tm_year, date.tm_yday, quality

        # Title does not match - should send email when this happens.
        raise TypeError('uhh... "%s" does not match any of our regex expressions.' % title)

    def _parse_tree(self):
        channel = self._root.find('channel')
        self._raw_updates = {'showrss': {
            'episodes': [],
            'source_link': channel.find('link').text
        }}

        for new_episode in channel.findall('item'):
            raw_title = new_episode.find('title').text
            try:
                season, episode, quality = self._extract_from_title(raw_title)
            except TypeError, error:
                err(str(error))
                continue

            title = new_episode.find('{%s}showname' % self.RSS_ENDPOINT).text
            self._raw_updates['showrss']['episodes'].append({
                'title': self.collection.build_title(title),
                'raw_title': raw_title,
                'link': new_episode.find('link').text,
                'season': season,
                'episode': episode,
                'quality': int(quality) if quality else None

                # TODO: Parse the published_date.
                #       format: Mon, 21 Apr 2014 02:40:02 +0000
                # 'published_date': new_episode.find('pubDate').text
            })

    def _is_tracked(self, title):
        """ Determines if episode belongs to a show that we are currently tracking.

            The tracked title is returned if the episode is tracked. None otherwise.
        """
        for tracked_title in self.collection.titles:
            # Title is a substring of the title and it should have a decent match ratio.
            if (tracked_title in title or title in tracked_title) and SequenceMatcher(title, tracked_title).ratio() > 0.6:
                return tracked_title

            # Title has a pretty damn high match ratio.
            if SequenceMatcher(None, title, tracked_title).ratio() > 0.9:
                return tracked_title

        return None

    def _append(self, episode, collection):
        if RSSSource._should_append(episode, collection):
            collection.append(episode)

    def _cleanse_updates(self):
        """ Removes duplicates and organises the XML response. """
        self._cleansed_updates = deepcopy(self._raw_updates)
        del self._cleansed_updates['showrss']['episodes']

        tracked_episodes, untracked_episodes = [], []
        for episode in self._raw_updates['showrss']['episodes']:
            tracked_title = self._is_tracked(episode['title'])
            t_collection = tracked_episodes if tracked_title else untracked_episodes
            self._append(dict(episode, **{'tracked_title': tracked_title}), t_collection)

        self._cleansed_updates['showrss']['tracked_episodes'] = tracked_episodes
        self._cleansed_updates['showrss']['untracked_episodes'] = untracked_episodes

    @property
    def raw_updates(self):
        """
        :returns: A dictionary of pre-processed updates from ShowRSS.
        :rtype: dict
        """
        return self._raw_updates

    @property
    def cleansed_updates(self):
        """
        :returns: A dictionary of post-processed updates from ShowRSS.
        :rtype: dict
        """
        return self._cleansed_updates

    @property
    def new_tracked_episodes(self):
        """
        :returns: A list of episodes that are currently recorded by :mod:`tvm`.
        :rtype: list

        .. note::
            There is a chance that the list of tracked episodes may contain duplicates.
        """
        return self._cleansed_updates['showrss']['tracked_episodes']

    @property
    def new_untracked_episodes(self):
        """
        :returns: A list of episodes that are currently not recorded by :mod:`tvm`.
        :rtype: list
        """
        return self._cleansed_updates['showrss']['untracked_episodes']

    @property
    def new_tracked_and_downloadable(self):
        """Filters out the list of tracked episodes and only returns a list of new tracked episodes.

        :returns: A list of **new** episodes that are currently recorded by :mod:`tvm`.
        :rtype: list

        .. note::
            It is recommended that this property should be called instead of :func:`new_tracked_episodes`.
        """
        def _downloadable():
            self._downloadable, keys = [], self.collection.collection.keys()
            for episode in self.new_tracked_episodes:
                key = (str(episode['season']), str(episode['episode']), episode['tracked_title'])
                if key not in keys:
                    self._downloadable.append(episode)

            self._downloadable = sorted(self._downloadable, key=lambda t: (t['title'], t['season'], t['episode']))
            return self._downloadable

        try:
            return self._downloadable
        except AttributeError:
            return _downloadable()

    @classmethod
    def _should_append(cls, episode, rss_collection):
        """ Determines if we should consider an episode worth downloading. """
        found_episode = None
        for stored_episode in rss_collection:
            if stored_episode['season'] == episode['season'] \
                    and stored_episode['episode'] == episode['episode'] \
                    and stored_episode['title'] == episode['title']:

                found_episode = stored_episode
                break

        if not found_episode:
            return True

        # If the quality is better than the duplicate, ignore duplicate.
        if found_episode['quality'] > episode['quality']:
            return False

        # Reached here means that:
        #   1) Duplicate has a better video quality.
        rss_collection.remove(found_episode)

        # TODO: _should_append should favor PROPER/REPACK versions.
        return True


def create_rss(cached_objects, collection):
    """Creates a single RSSSource by merging cached RSSSource objects. RSSSource objects
    are converted into JSON objects before they are cached.

    Merging `cached_objects` can cause conflicts due to duplicates and will automatically
    be `uniq` as a result.

    :param list cached_objects: A list of deserialized json objects.
    :param collection: An instance of the :class:`TVCollection` class.
    :type collection: :class:`TVCollection`
    """
    if not cached_objects:
        return None

    merged_tracked, merged_untracked = [], []
    for cached in cached_objects:
        for tracked in cached['showrss']['tracked_episodes']:
            if RSSSource._should_append(tracked, merged_tracked):
                merged_tracked.append(tracked)

        for untracked in cached['showrss']['untracked_episodes']:
            if RSSSource._should_append(untracked, merged_untracked):
                merged_untracked.append(untracked)

    return RSSSource(collection, {
        'showrss': {
            'source_link': 'http://showrss.info/',
            'tracked_episodes': merged_tracked,
            'untracked_episodes': merged_untracked
        }
    })
