#!/usr/bin/env python
# -*- coding: utf-8 -*-


class TVStatistics(object):
    def __init__(self, collection):
        """Many functions in :class:`TVStatistics` allow an optional `title` argument.
        When one is supplied, statistics will be generated based on the title.  When a
        `title` is not supplied, statistics will be based on the overall data.

        :param collection: An instance of the :class:`TVCollection` class.
        :type collection: :class:`TVCollection`
        """
        self.collection = collection

    def _gather_date_statistics(self, title=None):
        """ Provide a title to find date statistics on a specific show. """
        keys = [
            'last_updated',
            'first_recorded',
            'last_viewed',
            'first_viewed'
        ]

        # NOTE: Replace if we ever drop v2.6
        #       d = {key: value for (key, value) in sequence}
        date_statistics = dict((k, (None, None)) for k in keys)

        for episode, dates in self.collection.collection.iteritems():
            if title not in episode and title is not None:
                continue

            # dates = [[dates_viewed, ...], date_recorded]
            dates_viewed, date_recorded = dates

            # May contain None so we must prune those first -- sorts in ASC.
            dates_viewed = sorted([d for d in dates_viewed if d])

            if not date_statistics['last_updated'][0]:
                date_statistics['last_updated'] = (date_recorded, episode)
            if not date_statistics['first_recorded'][0]:
                date_statistics['first_recorded'] = (date_recorded, episode)
            if not date_statistics['last_viewed'][0] and dates_viewed:
                date_statistics['last_viewed'] = (dates_viewed[-1], episode)
            if not date_statistics['first_viewed'][0] and dates_viewed:
                date_statistics['first_viewed'] = (dates_viewed[0], episode)

            # Real comparisons where we actually have dates stored.
            if date_recorded >= date_statistics['last_updated'][0]:
                date_statistics['last_updated'] = (date_recorded, episode)
            if date_recorded <= date_statistics['first_recorded'][0]:
                date_statistics['first_recorded'] = (date_recorded, episode)

            # Not viewed... let's move on.
            if not dates_viewed:
                continue

            if dates_viewed[-1] >= date_statistics['last_viewed'][0]:
                date_statistics['last_viewed'] = (dates_viewed[-1], episode)
            if dates_viewed[0] <= date_statistics['first_viewed'][0]:
                date_statistics['first_viewed'] = (dates_viewed[0], episode)

        return date_statistics

    def first_viewed_episode(self, title=None):
        """Determines the first viewed episode.

        :param str title: (optional) The title of the TV show.

        .. note::
            See :func:`tvm.tvcollection.find_matching_titles` for the correct `title`.
        """
        return self._gather_date_statistics(title)['first_viewed']

    def first_recorded_episode(self, title=None):
        """Determines the first recorded episode.

        :param str title: (optional) The title of the TV show.

        .. note::
            See :func:`tvm.tvcollection.find_matching_titles` for the correct `title`.
        """
        return self._gather_date_statistics(title)['first_recorded']

    def last_viewed_episode(self, title=None):
        """Determines the last viewed episode.

        :param str title: (optional) The title of the TV show.

        .. note::
            See :func:`tvm.tvcollection.find_matching_titles` for the correct `title`.
        """
        return self._gather_date_statistics(title)['last_viewed']

    def last_updated_episode(self, title=None):
        """Determines the last updated episode.

        :param str title: (optional) The title of the TV show.

        .. note::
            See :func:`tvm.tvcollection.find_matching_titles` for the correct `title`.
        """
        return self._gather_date_statistics(title)['last_updated']

    def percentage_viewed(self, title=None):
        """Determines the percentage of viewed episodes, total episodes recorded and
        total episodes viewed.

        :param str title: (optional) The title of the TV show.
        :returns: A tuple in the form (percentage, no_recorded, no_viewed).
        :rtype: tuple

        .. note::
            See :func:`tvm.tvcollection.find_matching_titles` for the correct `title`.
        """
        if title:
            all_episodes, viewed_episodes = self.collection.find_all_episodes(title)
        else:
            all_episodes, viewed_episodes = self.collection.collection, self.collection.all_viewed_episodes

        try:
            percentage = round(float(len(viewed_episodes)) / len(all_episodes), 2) * 100
            return percentage, len(all_episodes), len(viewed_episodes)
        except ZeroDivisionError:
            return 0, 0, 0
