#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import os
from time import sleep

from tvm.commands import base

from tvm.api.config import config
from tvm.api.io import err, info, info2
from tvm.api.sources.rss import RSSSource, create_rss
from tvm.api.notification import Notifier

from tvm.cache.rss import RSSCache
from tvm.cache.notifications import NotificationCache


class CheckCommand(base.BaseCommand):
    def __init__(self, **arguments):
        super(self.__class__, self).__init__(**dict(arguments, **{'_ignore_errors': True}))

        self.helper = CheckHelper(self.collection)

        self._extact_args(arguments)
        self._exec()

    def _extact_args(self, arguments):
        self._without_cache = arguments.get('--without-cache', False)
        self._untracked = arguments.get('--untracked', False)
        self._with_ignored = arguments.get('--with-ignored', False)

        try:
            self._sleepn = int(arguments.get('--sleep', 0))
            if self._sleepn < 0:
                err('--sleep=<n> must be a positive integer.')
                exit(0)

        except ValueError:
            err('--sleep=<n>, "%s" is not a number.' % arguments['--sleep'])
            exit(0)

    def _exec(self):
        if self._sleepn:
            info('Sleeping %d second(s) before executing check command.' % self._sleepn)
            sleep(self._sleepn)

        rss = RSSSource(self.collection)
        if not self._without_cache:
            RSSCache.cache(rss)

        # Create a RSSSource using queued downloads from the cache directory.
        combined_rss = create_rss(RSSCache.load(), self.collection)

        # --without-cache gives combined_rss a None value.
        if combined_rss: rss = combined_rss

        self.helper.notify(rss, not self._without_cache)
        self.helper.print_updates(rss, show_untracked=self._untracked, show_ignored=not self._with_ignored)


class CheckHelper(base.BaseHelper):
    def __init__(self, collection):
        super(self.__class__, self).__init__(collection)

    def _notify_message(self, new_episodes):
        # Removes any duplicate titles in the notification popup.
        titles = list(set([t['tracked_title'] for t in new_episodes]))
        string_options = (len(new_episodes), ', '.join(titles))

        if len(new_episodes) == 1:
            return 'There\'s a new episode of %s to download.' % new_episodes[0]['tracked_title']
        return '%d new episodes to download: %s.' % string_options

    def notify(self, rss, cache_notification=True):
        """ Set `cache_notification=False` to not cache notification. """
        notification = NotificationCache.load()

        # TODO: The check to notify can be improved.
        #   - Even if the notification matches rss.new_tracked_and_downloadable, if it exceeds
        #     12hrs then it should notify anyway.
        #   - If I get a notification for x, y and z. I then download z. The next
        #     notification will be x and y only. This means a notification message will appear.
        #
        #     Should this be accounted for or should this be a "feature"? The "feature" being
        #     that there's more episodes to download (missed) and the user should know this.
        if len(rss.new_tracked_and_downloadable) > 0 and notification != rss.new_tracked_and_downloadable:
            Notifier.notify(self._notify_message(rss.new_tracked_and_downloadable),
                            title='TVM - TV Manager',
                            sound='DEFAULT')

            if cache_notification:
                NotificationCache.cache(rss.new_tracked_and_downloadable)

    def print_updates(self, rss, show_untracked=False, show_ignored=False):
        info('The following are the new episodes found:')

        # TODO: Do this ignore check for the notify function.
        amount_ignored = 0
        for tracked in rss.new_tracked_and_downloadable:
            if show_ignored and tracked['tracked_title'] in config.ignore_titles:
                amount_ignored += 1
                continue

            info2('%s,%s,"%s"' % (tracked['season'], tracked['episode'], tracked['tracked_title']))

        if show_untracked:
            info('-' * 79)
            for untracked in rss.new_untracked_episodes:
                info2(untracked['raw_title'])

        options = (
            len(rss.new_tracked_and_downloadable) - amount_ignored,
            len(rss.new_untracked_episodes),
            amount_ignored
        )

        info('Tracked: %d | Untracked: %d | Ignored: %d.' % options)
        info('Total cached JSON objects: %d' % len(os.listdir(config.cache_path)))
