#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

from datetime import datetime
import random
import re
import sys
import tty
import subprocess

from tvm.commands import base

from tvm.api.config import config
from tvm.api.io import err, info, info2, ok
from tvm.api.io import print_matched_status
from tvm.api.io import build_episode_path

from tvm.api.dates import time_now_str, time_to_str, datetime_to_time
from tvm.api.sources.video import VideoSource
from tvm.api.tvstatistics import TVStatistics


class PlayCommand(base.BaseCommand):
    def __init__(self, **arguments):
        super(self.__class__, self).__init__(**dict(arguments, **{'_ignore_errors': True}))

        self.helper = PlayHelper(self.collection)
        self._extact_args(arguments)
        self._exec()

    def _extact_args(self, arguments):
        self._search_query = ' '.join(arguments.get('<title>', []))
        self._search_query = self.helper.extract_episode(self._search_query)

        self._search_title = None
        self._has_search_query, self._is_single_episode = False, False
        if isinstance(self._search_query, tuple):  # Specific episode given.
            self._is_single_episode = True
            self._has_search_query = True

            # season, episode, title.
            self._search_title = self._search_query[-1]
        elif self._search_query:  # Non-specific episode.
            self._has_search_query = True
            self._search_title = self.collection.build_title(self._search_query)

        self._no_open = arguments.get('--no-open', False)
        self._no_record = arguments.get('--no-record', False)
        self._undo = arguments.get('--undo', False)
        self._random = arguments.get('--random', False)
        self._iter = arguments.get('--iter', False)

    def _exec(self):
        title = self.collection.find_matching_titles(self._search_title)
        if self._has_search_query:
            print_matched_status(title)  # Will terminate if None is found.

        if self._undo:
            removed_episode = self.helper.undo_last_played(title)
            if not removed_episode:
                return err("Unable to --undo the episode.")
            return ok("'%s' --undo was successful." % removed_episode)

        if self._random:
            return self.helper.play_random_episode(title, not self._no_open, not self._no_record)

        if self._iter:
            return self.helper.play_iter_selection(title, not self._no_open, not self._no_record)

        # Handle playing the "specific" episode.
        if self._is_single_episode:
            episode = tuple(self._search_query[:-1]) + (title, )
            return self.helper.play_episode(episode, not self._no_open, not self._no_record)

        # Handle playing the "next" episode.
        next_to_play = self.helper.next_to_play(title)
        if not next_to_play:
            return info('No new episodes for "%s".' % title)

        return self.helper.play_episode(next_to_play, not self._no_open, not self._no_record)


class PlayHelper(base.BaseHelper):
    def __init__(self, collection):
        super(self.__class__, self).__init__(collection)

    def play_episode(self, episode, should_open=True, should_record=True):
        """ `episode` => season, episode, title """
        if should_open:
            info('Attempting to play episode, "%s".' % ','.join(episode))
        else:
            info('Not opening the episode "%s".' % ','.join(episode))

        episode_path = build_episode_path(episode)
        if should_open and not VideoSource.open(episode_path):
            err('Failed to open the episode, "%s".' % ','.join(episode))
            should_record = False

        if should_record:
            self.update_records_with(episode)
        else:
            info('Not recording "%s" as viewed.' % ','.join(episode))

        self.print_info(episode)
        ok('Successfully opened episode, "%s".' % ','.join(episode))

    def play_random_episode(self, title=None, should_open=True, should_record=True):
        episode = self._get_selection_pool(title).next()
        return self.play_episode(episode, should_open, should_record)

    def play_iter_selection(self, title, should_open=True, should_record=True):
        info('Starting random selection simulation.')
        info('- Press "Enter" or "y" to play, "any key" to skip and "q" to exit.')

        tty.setcbreak(sys.stdin)
        for episode in self._get_selection_pool(title):
            info2('Play "%s" [y/q/(any key)]? ' % ','.join(episode), end='')
            response = sys.stdin.read(1).lower().strip('\n')
            print(response)  # Display what the user typed after the query.

            if response == 'y' or not response:  # Enter or 'y'
                return self.play_episode(episode, should_open, should_record)
            elif response == 'q':
                return info('Leaving random selection simulation.')

        info('There are no more episodes to randomly select from. Exiting.')

    def _get_selection_pool(self, title=None):
        def _create_selection_pool():
            if title:
                # Return the all episodes as the selection pool if we're up to date with the show.
                recorded, viewed = self.collection.find_all_episodes(title)
                if self.collection.viewed_up_to_date(title):
                    return recorded
                return viewed

            # A random selection on all recorded shows that have been viewed.
            return [episode[0] for episode in self.collection.all_viewed_episodes]

        selection_pool = _create_selection_pool()
        random.shuffle(selection_pool)

        for e in selection_pool: yield e

    def update_records_with(self, episode):
        info('Recording "%s" as viewed.' % ','.join(episode))
        self.collection.save([(datetime.now(), ) + episode], save_type='viewed')

    def next_to_play(self, title):
        """ The `title` must be an exact title. @see find_matching_titles to find an exact match. """
        all_episodes, viewed_episodes = map(lambda e: self.collection.sort_episodes(e, desc=False),
                                            self.collection.find_all_episodes(title))

        # We have never seen any episodes yet.
        if not viewed_episodes:
            return all_episodes[0]  # First episode.

        last_viewed_episode = viewed_episodes[-1]

        # The most recently viewed episode is the last episode added.
        if all_episodes[-1] == last_viewed_episode:
            return None

        index = all_episodes.index(last_viewed_episode)
        return all_episodes[index + 1]  # Next episode to watch.

    def extract_episode(self, query):
        """ Extracts the (season, episode, title) from the `query`. """
        def _extract(pattern):
            try:
                return pattern.match(query).groups()
            except AttributeError:
                pass

        # title, season, episode
        extracted = _extract(re.compile(r'(.+)\s+s(\d+)\s*e(\d+)$'))
        if isinstance(extracted, tuple):
            return extracted[1], extracted[2], self.collection.build_title(extracted[0])

        # title, episode, season
        extracted = _extract(re.compile(r'(.+)\s+e(\d+)\s*s(\d+)$'))
        if isinstance(extracted, tuple):
            return extracted[2], extracted[1], self.collection.build_title(extracted[0])

        # season, episode, title
        extracted = _extract(re.compile(r's(\d+)\s*e(\d+)\s+(.+)$'))
        if isinstance(extracted, tuple):
            return extracted[0], extracted[1], self.collection.build_title(extracted[2])

        # episode, season, title
        extracted = _extract(re.compile(r'e(\d+)\s*s(\d+)\s+(.+)$'))
        if isinstance(extracted, tuple):
            return extracted[1], extracted[0], self.collection.build_title(extracted[2])

        # title, episode (no season, default: 0)
        extracted = _extract(re.compile(r'(.+)\s+e(\d+)$'))
        if isinstance(extracted, tuple):
            return '0', extracted[1], self.collection.build_title(extracted[0])

        # episode, title (no season, default: 0)
        extracted = _extract(re.compile(r'e(\d+)\s+(.+)$'))
        if isinstance(extracted, tuple):
            return '0', extracted[0], self.collection.build_title(extracted[1])

        return query  # Return the original query if nothing was found.

    def undo_last_played(self, title=None):
        if not len(self.collection.all_viewed_episodes):  # Nothing has yet to be viewed.
            return None

        # TODO: Will have to improve on this to support specific show undos.
        #   - This may involve having to load the TVCollection, make changes then re-write it.

        proc = subprocess.Popen(['tail', '-n-1', config.viewed_path], stdout=subprocess.PIPE)
        removed_episode = proc.stdout.read().strip('\n')

        proc = subprocess.Popen(['head', '-n-1', config.viewed_path], stdout=subprocess.PIPE)
        updated_file = proc.stdout.read()

        with open(config.viewed_path, 'w') as out_file:
            out_file.write(updated_file)

        return removed_episode

    def print_info(self, entry):
        season, episode, title = entry

        episode_path = build_episode_path(entry)
        if episode_path:
            duration = VideoSource.duration(episode_path)
            info2('%s -- %s (duration: %s minutes).' % (
                time_now_str(),
                time_to_str(datetime_to_time(datetime.now() + duration)),
                duration.seconds / 60)
            )

        # Display the percentage of viewed episodes.
        statistics = TVStatistics(self.collection)
        percentage, recorded, viewed = statistics.percentage_viewed(title)

        info2('%d%% (%d/%d) total views.' % (percentage, viewed, recorded))
