#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""tvm.py

usage:
    tvm.py check [--sleep=<n> --without-cache --untracked --with-ignored]
    tvm.py doctor [--repository --records]
    tvm.py generate <path> [--append --force]
    tvm.py ls [[<title>... [--expand]]|--new]
    tvm.py play [<title>...] (--random|--iter) [--no-open --no-record]
    tvm.py play [<title>...] --undo
    tvm.py play <title>... [--no-open --no-record]

commands:
    check            checks for new tv shows to download.
    doctor           checks for syntax and semantic errors in the recorded files.
    generate         generates records to be added in shows_added with <path>.
    ls               lists all tv shows recorded.
    play             plays the next episode in the given tv show <title>.

check:
    --sleep=<n>      sleeps n seconds before performing the check [default: 0].
    --without-cache  checks but does not store results in cache.
    --untracked      shows untracked episodes that can be downloaded.
    --with-ignored   hides all new episodes for shows that I've ignored

doctor:
    --records        checks if physical files exists in the records.
    --repository     checks if entries in record have a physical counterpart.

generate:
    -a --append      appends stdout to records.
    -f --force       pre-tracked records aren't generated, force generates them.

ls:
    -e --expand      completed and pending seasons aren't collapsed.
    -n --new         only display TV shows with unviewed episodes.

play:
    --no-open        records but doesn't open the episode.
    --no-record      opens but doesn't record the episode.
    -r --random      randomly selects a show to play.
    -u --undo        undo the most previously played episode.
    -i --iter        iterates over selection pool, randomly suggesting shows.

arguments:
    <path>           directory containing tv shows.
    <title>          title of the tv show.

options:
    -h --help        shows this.
    -v --version     shows version.
"""
from __future__ import print_function

from os import chdir, path
from docopt import docopt
from inspect import getmembers, isclass

try:
    from commands import *
    from . import __version__
except ImportError:
    import traceback
    traceback.print_exc()

    print('Development mode: Run with -m `python -m tvm/tvm [command] [arguments]`.')
    print('Alternatively, run python setup.py develop; tvm -h')
    exit(0)


def main():
    arguments = docopt(__doc__, version=__version__)

    # Set the current directory to the directory of this file.
    chdir(path.dirname(path.abspath(__file__)))

    # Dynamically load corresponding command class given command name.
    for command, value in arguments.iteritems():
        if command.startswith('--'):  # Options.
            continue

        if command.startswith('<'):  # Positional arguments.
            continue

        if not value:  # Not a command that was called.
            continue

        _, klass = getmembers(globals()[command], isclass)[0]
        klass(**arguments)

if __name__ == '__main__':
    main()
